/* Beispiel fuer Java-Datenstrukturen
   "Warengruppe" zur Vorlesung "Softwaretechnologie".
   * Firmen bieten Warengruppen (Mengen von Artikeln) in Katalogen an.
   * Eine Bestellung kann sich auf Artikel einer Warengruppe beziehen.
   * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2013 

   * Duplikate von Artikeln dürfen nicht existieren.
   * Diese Version des Beispiels "Warengruppe" verwendet HashSet.
   * Neben der Mengeneigenschaft wird der Effekt von Objektidentitaeten demonstriert.
   * @Version 2.0 Generics
   */


import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

class Artikel {

    private String name;
    private int preis;
  
    public Artikel (String name, int preis) {
	this.name = name;
	this.preis = preis;
    }
  
    public int preis() {
	return preis;
    }
  
    public String toString() {
	return name+"("+preis+")";
    }
}


class Warengruppe0 {

    private String name;
    private String lagerplatz;
    private Set<Artikel> inhalt;
  
    public Warengruppe0 (String name, String lagerplatz) {
	this.name = name;
	this.lagerplatz = lagerplatz;
	this.inhalt = new HashSet<Artikel>();
    }
        
    public void add (Artikel a) {
	inhalt.add(a);
    }
  
    public int anzahl() {
	return inhalt.size();
    }
  
    public String toString() {
	String s = "Warengruppe "+name+", Lagerplatz "+lagerplatz+"\n";
	Iterator<Artikel> it = inhalt.iterator();
	while (it.hasNext()) {
	    s += " "+it.next();
	};
	s += "\nAnzahl Artikel in Warengruppe: "+inhalt.size()+"\n";
	return s;
    }
  
  
    public static void main (String[] args) {
      
	Warengruppe0 w1 = new Warengruppe0("Moebel","L1");
	w1.add(new Artikel("Tisch",200));
	w1.add(new Artikel("Stuhl",100));
	w1.add(new Artikel("Schrank",300));
	w1.add(new Artikel("Tisch",200));
	System.out.println(w1);
    
	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
  
	Warengruppe0 w2 = new Warengruppe0("Moebel","L2");
	w2.add(tisch);
	w2.add(stuhl);
	w2.add(schrank);
	w2.add(tisch);  // Duplikat-Eintrag wird ignoriert
	System.out.println(w2);

    }
  
}
