/* Beispiel fuer Java-Datenstrukturen
   "Warengruppe" zur Vorlesung "Softwaretechnologie".
   * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2013 

   * Diese Version des Beispiels "Warengruppe" verwendet HashSet.
   * Es wird gezeigt, wie nach Ueberdefinition von equals und hashCode
   * Objekte verschiedener Identitaet als gleich betrachtet werden.
   * @Version 2.0 Generics
 */


import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

class Artikel {

    private String name;
    private int preis;
  
    public Artikel (String name, int preis) {
	this.name = name;
	this.preis = preis;
    }
  
    public int preis() {
	return preis;
    }
  
    /** Domänenspezifische Gleichheit für Artikel. 
     * Object.equals() wird überschrieben. Funktioniert. 
     * Leider muss equals auf Object definiert werden, da sonst Object.equals
     * nicht überschrieben wird. Das ist eine ärgerliche Einschränkung des
     * Java-Typsystems. 
     */
    public boolean equals (Object o) {
	return ((Artikel)o).name.equals(this.name) && 
	    ((Artikel)o).preis == this.preis;
    }
  
    /** Domänenspezifische Gleichheit für Artikel.  (funktioniert nicht!)
    */
    /*
    public boolean equals (Artikel a) {
	return a.name.equals(this.name) && 
	    a.preis == this.preis;
    }
    */
  
    public int hashCode () {
	return name.hashCode();
    }
  
    public String toString() {
	return name+"("+preis+")";
    }
}


class Warengruppe1 {

    private String name;
    private String lagerplatz;
    private Set<Artikel> inhalt;
  
    public Warengruppe1 (String name, String lagerplatz) {
	this.name = name;
	this.lagerplatz = lagerplatz;
	this.inhalt = new HashSet<Artikel>();
    }
        
    public void add (Artikel a) {
	inhalt.add(a);
    }
  
    public int anzahl() {
	return inhalt.size();
    }
  
    public String toString() {
	String s =
	    "Warengruppe "+name+", Lagerplatz "+lagerplatz+"\n";
	Iterator<Artikel> it = inhalt.iterator();
	while (it.hasNext()) {
	    s += " "+it.next();
	};
	s += "\nAnzahl Artikel in Warengruppe: "+inhalt.size()+"\n";
	return s;
    }
  
  
    public static void main (String[] args) {
      
	Warengruppe1 w1 = new Warengruppe1("Moebel","L1");
	w1.add(new Artikel("Tisch",200));
	w1.add(new Artikel("Stuhl",100));
	w1.add(new Artikel("Schrank",300));
	w1.add(new Artikel("Tisch",500));
	System.out.println(w1);
    
	Artikel tisch = new Artikel("Tisch",200);
	Artikel stuhl = new Artikel("Stuhl",100);
	Artikel schrank = new Artikel("Schrank",300);
  
	Warengruppe1 w2 = new Warengruppe1("Moebel","L2");
	w2.add(tisch);
	w2.add(stuhl);
	w2.add(schrank);
	w2.add(tisch); // Doppeleintrag
	System.out.println(w2);
    }
  
}
