/* Beispiel fuer Java-Datenstrukturen
 * "Warengruppe" zur Vorlesung "Softwaretechnologie".
 * (c) Heinrich Hussmann, Uwe Aßmann, TU Dresden, 2013 

 * Diese Version des Beispiels "Warengruppe" verwendet als Oberklasse HashSet.
 * Es wird eine Loesung mit Vererbung statt Delegation demonstriert.
 * @Version 2.0 Generics
 */


import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

class Artikel {

  private String name;
  private int preis;
  
  public Artikel (String name, int preis) {
    this.name = name;
    this.preis = preis;
  }
  
  public int preis() {
    return preis;
  }
  
    /** Noch eine Form von Gleichheit: Name und Preis müssen überein stimmen.
     */
  public boolean equals (Object o) {
    return ((Artikel)o).name == this.name && 
           ((Artikel)o).preis == this.preis;
  }
  
  public int hashCode () {
    return name.hashCode();
  }
  
  public String toString() {
    return name+"("+preis+")";
  }
}


class Warengruppe2<E> extends HashSet<E> {

  private String name;
  private String lagerplatz;
  
  public Warengruppe2 (String name, String lagerplatz) {
    super();
    this.name = name;
    this.lagerplatz = lagerplatz;
  }
  
  public int anzahl() {
    return this.size();
  }
  
  public String toString() {
    String s = "Warengruppe "+name+", Lagerplatz "+lagerplatz+"\n";
    Iterator<E> it = iterator();
    while (it.hasNext()) {
      s += " "+it.next();
    };
    s += "\nAnzahl Artikel in Warengruppe: "+size()+"\n";
    return s;
}
  
  
  public static void main (String[] args) {
      
    Warengruppe2<Artikel> w1 = new Warengruppe2<Artikel>("Moebel","L1");
    w1.add(new Artikel("Tisch",200));
    w1.add(new Artikel("Stuhl",100));
    w1.add(new Artikel("Schrank",300));
    w1.add(new Artikel("Tisch",100)); // Doppeleintrag wird NICHT ignoriert
				      // (Name und Preis)
    System.out.println(w1);
    
    Artikel tisch = new Artikel("Tisch",200);
    Artikel stuhl = new Artikel("Stuhl",100);
    Artikel schrank = new Artikel("Schrank",300);
  
    Warengruppe2<Artikel> w2 = new Warengruppe2<Artikel>("Moebel","L2");
    w2.add(tisch);
    w2.add(stuhl);
    w2.add(schrank);
    w2.add(tisch); // Doppeleintrag wird ignoriert
    System.out.println(w2);
  }
  
}
